﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Text;
using System.Runtime.InteropServices;

/// <summary>
/// ストリーム型DBクラス用名前空間
/// </summary>
namespace EgSDB
{
    /// <summary>
    /// 定数クラス
    /// </summary>
    public static class SDBCONST
    {

        /// <summary>
        /// SDBファイル単位サイズ(バイト単位)：デフォルト値
        /// </summary>
        public const Int64 FRAGMENT_SIZE = 0x20000000;

        /// <summary>
        /// SDBファイル単位サイズ(バイト単位)：最小値
        /// </summary>
        /// <remarks>
        /// SDBファイル単位サイズ最小値は、とりあえず1KBを定義します。
        /// </remarks>
        public const Int64 MIN_FRAGMENT_SIZE = 1024;

        /// <summary>
        /// 記録容量(バイト単位)：デフォルト値
        /// </summary>
        public const Int64 CAPACITY_SIZE = 0x100000000;

        /// <summary>
        /// 記録容量(倍率)：最小値
        /// </summary>
        /// <remarks>
        /// 記録容量最小値は、 FragmentSize * MIN_CAPACITY_RATE で算出します。
        /// </remarks>
        public const Int64 MIN_CAPACITY_RATE = 2;

        /// <summary>
        /// ファイルヘッダ識別子（"SDAT"をASCIIに変換）：固定値
        /// </summary>
        public const String FILEHEADER_ID = "SDAT";

        /// <summary>
        /// ファイルヘッダサイズ：固定値
        /// </summary>
        public const Byte FILEHEADER_SIZE = 32;

        /// <summary>
        /// ファイル拡張子
        /// </summary>
        public const String FILE_EXT = "SDT";

        /// <summary>
        /// ファイルサフィックス
        /// </summary>
        public const String FILE_SUFFIX = "SDAT";

    }

    /// <summary>
    /// メッセージ番号
    /// </summary>
    public enum SDBMsgNo : Int32
    {
        /// <summary>
        /// レコード構成生成
        /// </summary>
        CreateFormat = 20000,

        /// <summary>
        /// レコードバイナリ追加
        /// </summary>
        InsertBinary = 20001,
    }

    /// <summary>
    /// SDBItem
    /// </summary>
    /// <remarks>
    /// SDBファイルに関するデータを管理するクラス    
    /// </remarks>
    public class SDBItem
    {
        /// <summary>
        /// ファイル名
        /// </summary>
        public String FileName
        {
            get { return m_srFileName; }
            set { m_srFileName = value; }
        }
        private String m_srFileName;

        /// <summary>
        /// ファイルサイズ
        /// </summary>
        public Int64 FileSize
        {
            get { return m_loFileSize; }
            set { m_loFileSize = value; }
        }
        private Int64 m_loFileSize;

        /// <summary>
        /// 更新日時
        /// </summary>
        public DateTime UpdateTime
        {
            get { return m_dtUpdateTime; }
            set { m_dtUpdateTime = value; }
        }
        private DateTime m_dtUpdateTime;

        /// <summary>
        /// ファイルヘッダ
        /// </summary>
        public FileHeader FileHeader
        {
            get { return m_stFileHeader; }
            set { m_stFileHeader = value; }
        }
        public FileHeader m_stFileHeader;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public SDBItem()
        {
            // 
        }

        /// <summary>
        /// デストラクタ
        /// </summary>
        ~SDBItem()
        {
            // 
        }
    }

    /// <summary>
    /// ファイルヘッダ
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack =1, Size =SDBCONST.FILEHEADER_SIZE)]
    public struct FileHeader
    {
        /// <summary>
        /// ファイル識別子
        /// </summary>
        /// <remarks>
        /// "SDB1"をASCII変換した値
        /// </remarks>
        public UInt32 Id
        {
            get { return m_dwId; }
            set { m_dwId = value; }
        }
        private UInt32 m_dwId;

        /// <summary>
        /// ファイル識別子：収納位置
        /// </summary>
        public static UInt32 PosId = 0;

        /// <summary>
        /// ヘッダサイズ
        /// </summary>
        public Byte HeadSize
        {
            get { return m_byHeadSize; }
            set { m_byHeadSize = value; }
        }
        private Byte m_byHeadSize;

        /// <summary>
        /// ヘッダサイズ：収納位置
        /// </summary>
        /// <remarks>
        /// 収納位置 = Id + Idの変数サイズ
        /// </remarks>
        public static UInt32 PosHeadSize = PosId + sizeof(UInt32);

        /// <summary>
        /// 最新のレコード構成番号
        /// </summary>
        public Int32 FormatNumber
        {
            get { return m_inFormatNumber; }
            set { m_inFormatNumber = value; }
        }
        private Int32 m_inFormatNumber;

        /// <summary>
        /// 最新のレコード構成番号：収納位置
        /// </summary>
        /// <remarks>
        /// 収納位置 = ヘッダサイズ + ヘッダサイズの変数サイズ
        /// </remarks>
        public static UInt32 PosFormatNumber = PosHeadSize + sizeof(Byte);

        /// <summary>
        /// 書き込み位置
        /// </summary>
        public UInt32 AddPointer
        {
            get { return m_dwAddPointer; }
            set { m_dwAddPointer = value; }
        }
        private UInt32 m_dwAddPointer;

        /// <summary>
        /// 書き込み位置：収納位置
        /// </summary>
        /// <remarks>
        /// 収納位置 = 最新のレコード構成番号 + 最新のレコード構成番号の変数サイズ
        /// </remarks>
        public static UInt32 PosAddPointer = PosFormatNumber + sizeof(Int32);

        /// <summary>
        /// 初期化
        /// </summary>
        public void Init()
        {
            
            Id = BitConverter.ToUInt32(Encoding.GetEncoding("Shift_JIS").GetBytes(SDBCONST.FILEHEADER_ID), 0);
            HeadSize = SDBCONST.FILEHEADER_SIZE;
            FormatNumber = 0;
            AddPointer = 0;
        }

    }

    /// <summary>
    /// 文字列
    /// </summary>
    public class SDBString
    {
        /// <summary>
        /// サイズ
        /// </summary>
        public byte Size
        {
            get { return m_bySize; }
            set { m_bySize = value; }
        }
        private byte m_bySize;

        /// <summary>
        /// 文字列
        /// </summary>
        public byte[] Val
        {
            get { return m_byVal; }
            set { m_byVal = value; }
        }
        private byte[] m_byVal;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="byData">文字列</param>
        public SDBString(byte[] byData)
        {
            // サイズの設定
            Size = (byte)byData.Length;
            // 文字列の設定
            Val = new byte[Size];
            Array.Copy(byData, Val, Size);
        }

        /// <summary>
        /// 長さ取得
        /// </summary>
        /// <returns>長さ</returns>
        public UInt16 GetLength()
        {
            // 長さは文字列サイズ込みの長さです。
            return (UInt16)(sizeof(Byte) + Val.Length);
        }

        /// <summary>
        /// バイト配列に変換
        /// </summary>
        /// <returns>バイト配列</returns>
        public Byte[] GetBytes()
        {
            Byte[] byData = new byte[GetLength()];

            /// [0]         文字数
            byData[0] = Size;
            /// [1] .. [n]  文字列
            Array.Copy(Val, 0, byData, 1, Size);

            return byData;
        }
    }

    /// <summary>
    /// タグ情報
    /// </summary>
    public class SDBTag
    {

        /// <summary>
        /// タグ名
        /// </summary>
        /// <remarks>
        /// バイナリファイルに書き込む際は、以下のバイト配列に変換して書き込む
        /// [0]         文字数
        /// [1] .. [n]  文字列
        /// </remarks>
        public SDBString Name
        {
            get { return m_clName; }
            set { m_clName = value; }
        }
        private SDBString m_clName;

        /// <summary>
        /// タグ名を取得
        /// </summary>
        /// <returns>タグ名</returns>
        public String GetName()
        {
            return Encoding.GetEncoding("Shift_JIS").GetString(Name.Val);
        }

        /// <summary>
        /// タグタイプ
        /// </summary>
        public Byte Type
        {
            get { return m_byType; }
            set { m_byType = value; }
        }
        private Byte m_byType;

        /// <summary>
        /// タグサイズ
        /// </summary>
        public Byte Size
        {
            get { return m_bySize; }
            set { m_bySize = value; }
        }
        private Byte m_bySize;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public SDBTag()
        {
            Name = null;
            Type = 0;
            Size = 0;
        }

    }


    /// <summary>
    /// レコード構成
    /// </summary>
    public class SDBRecordFormat
    {
        /// <summary>
        /// レコード構成番号
        /// </summary>
        public Int32 Number
        {
            get { return m_inNumber; }
            set { m_inNumber = value; }
        }
        private Int32 m_inNumber;

        /// <summary>
        /// レコードサイズ
        /// </summary>
        public UInt16 Size
        {
            get { return m_woSize; }
            set { m_woSize = value; }
        }
        private UInt16 m_woSize;

        /// <summary>
        /// タグ数
        /// </summary>
        public UInt16 TagCount
        {
            get { return m_woTagCount; }
            set { m_woTagCount = value; }
        }
        private UInt16 m_woTagCount;

        /// <summary>
        /// 登録状態 false=未登録 / true=登録済み
        /// </summary>
        public bool Configure
        {
            get { return m_boConfigure; }
            set { m_boConfigure = value; }
        }
        private bool m_boConfigure;

        /// <summary>
        /// タグ情報
        /// </summary>
        public SDBTag[] Tags;

        /// <summary>
        /// 初期化
        /// </summary>
        /// <param name="woTagCount">タグ数</param>
        public void Init(UInt16 woTagCount)
        {
            // レコード構成番号の設定
            Number = 0;
            // タグ数の設定
            TagCount = woTagCount;
            // レコードサイズの設定
            Size = 0;
            Size += (UInt16)Marshal.SizeOf(Number);
            Size += (UInt16)Marshal.SizeOf(Size);
            Size += (UInt16)Marshal.SizeOf(TagCount);
            // タグ情報の生成
            Tags = new SDBTag[TagCount];
            for (var i = 0; i < Tags.Length; i++)
            {
                Tags[i] = new SDBTag();
            }
            // レコード状態
            Configure = false;
        }

        /// <summary>
        /// タグ情報の生成
        /// </summary>
        /// <param name="dwIndex">インデックス</param>
        /// <param name="srName">タグ名</param>
        /// <param name="byType">タグタイプ</param>
        /// <param name="bySize">タグサイズ</param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public Int32 AddTags(UInt16 dwIndex, string srName, byte byType, byte bySize)
        {
            // インデックス判定
            if (dwIndex >= TagCount)
            {
                // インデックスエラー
            }

            // 指定したインデックスにタグ情報を追加

            // タグ名の設定
            // タグ名をバイト配列に変換
            byte[] byTagName = Encoding.GetEncoding("Shift_JIS").GetBytes(srName);
            // 長さの計算: タグ名:文字数 + タグ名:文字列
            Int32 inTagLen = sizeof(Byte) + byTagName.Length;
            // タグ名の生成
            Tags[dwIndex].Name = new SDBString(byTagName);
            // レコードサイズの設定
            Size += Tags[dwIndex].Name.GetLength();

            // タグタイプの設定
            Tags[dwIndex].Type = byType;
            // レコードサイズの設定
            Size += (UInt16)Marshal.SizeOf(Tags[dwIndex].Type);

            // タグサイズの設定
            Tags[dwIndex].Size = bySize;
            // レコードサイズの設定
            Size += (UInt16)Marshal.SizeOf(Tags[dwIndex].Size);



            return (Int32)SDBErr.Ok;
        }

    }



}
