﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.IO;
using System.IO.MemoryMappedFiles;
using System.Runtime.InteropServices;
using System.Diagnostics;

using RTedge;       // RT-edge system API


/// <summary>
/// ストリーム型DBクラス用名前空間
/// </summary>
namespace EgSDB
{

    /// <summary>
    /// SDBManager
    /// </summary>
    /// <remarks>
    /// ストリーム型DB機能のメインとなるクラスで、SDBファイル収納パスの管理、
    /// SDBファイルの管理などを行います。
    /// </remarks>
    public class SDBManeger
    {

        /// <summary>
        /// サービスインジケータ名
        /// </summary>
        private EgSDBIndName m_clIndName;

        /// <summary>
        /// サービスインジケータ値
        /// </summary>
        private EgSDBIndVal m_clIndVal;

        /// <summary>
        /// プロパティタグ名
        /// </summary>
        private EgSDBTagName m_clTagName;

        /// <summary>
        /// プロパティタグ値
        /// </summary>
        private EgSDBTagVal m_clTagVal;

        /// <summary>
        /// サービス名(24バイト) xmlから取得
        /// </summary>
        private string ServiceRealName = "EgSDAT";//デフォルトEgSDAT
        /// <summary>
        /// 最新のエラーコード取得
        /// </summary>
        /// <returns>最新のエラーコード</returns>
        public Int32 fnGetLastErr()
        {
            return m_clTagVal.m_inLastErr;
        }

        /// <summary>
        /// 自動起動設定取得
        /// </summary>
        /// <returns>自動起動設定</returns>
        public bool fnGetAutoRun()
        {
            return m_clTagVal.m_boAutoRun;
        }

        /// <summary>
        /// オンメモリDB(MemoryMappedFile)
        /// </summary>
        private MemoryMappedFile m_SDBMmf = null;

        /// <summary>
        /// オンメモリDBView
        /// </summary>
        private MemoryMappedViewStream m_SDBView = null;

        /// <summary>
        /// edge APIのインスタンス
        /// </summary>
        private edge_API m_EgApi = null;

        /// <summary>
        /// SDBItemコレクション
        /// </summary>
        /// <remarks>
        /// SDBファイルをリストにして管理します。
        /// </remarks>
        private List<SDBItem> m_SDBList;

        /// <summary>
        /// レコード構成コレクション 
        /// </summary>
        /// <remarks>
        /// DBコンテナ実行中に追加したレコード構成をリストで管理します。
        /// このコレクションでは、前回実行中に使用したレコード構成は管理しません。
        /// また前回実行中のレコード構成と今回実行中のレコード構成が同じ場合は、
        /// sdbファイルに新規にレコード構成を追加します。
        /// </remarks>
        private List<SDBRecordFormat> m_Record;


        /// <summary>
        /// SDBファイル収納パス
        /// </summary>
        public String StorePath
        {
            get { return m_srStorePath; }
            set
            {
                m_srStorePath = value;
                // もしフォルダがない場合は生成
                if (!Directory.Exists(m_srStorePath))
                {
                    // フォルダないため生成
                    Directory.CreateDirectory(m_srStorePath);
                }
            }
        }
        private String m_srStorePath;


        /// <summary>
        /// SDBファイル単位サイズ(バイト単位)
        /// </summary>
        public Int64 FragmentSize
        {
            get { return m_loFragmentSize; }
            set { m_loFragmentSize = value; }
        }
        private Int64 m_loFragmentSize;


        /// <summary>
        /// 記録容量(バイト単位)
        /// </summary>
        public Int64 CapacitySize
        {
            get { return m_loCapacitySize; }
            set { m_loCapacitySize = value; }
        }
        private Int64 m_loCapacitySize;

        /// <summary>
        /// コレクタ受信待ちスレッドキャンセル処理用変数
        /// </summary>
        private CancellationTokenSource m_thCollectorTS = null;

        /// <summary>
        /// 自身のプログラム名
        /// </summary>
        private string m_srMyName = "";

        /// <summary>
        /// ファイルクローズ要因
        /// </summary>
        /// <remarks>
        /// ファイルクローズ要因によってファイル削除が異なるために定義
        /// </remarks>
        public enum CloseFileFactor : Int32
        {
            /// <summary>
            /// ファイル切替で発生した
            /// </summary>
            ChangeFile = 0,
            /// <summary>
            /// コンテナ終了で発生した
            /// </summary>
            ExitContainer,
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="srMyName">自身のプログラム名</param>
        public SDBManeger(string srMyName)
        {
            // 自身のプログラム名の設定
            m_srMyName = srMyName;

            // SDBファイルリストの初期化
            m_SDBList = new List<SDBItem>();

            // レコード構成リストの初期化
            m_Record = new List<SDBRecordFormat>();

            // RT-edge APIクラス初期化
            m_EgApi = new edge_API();

            // サービスインジケータ初期化
            m_clIndName = new EgSDBIndName(srMyName);
            m_clIndVal = new EgSDBIndVal();

            // サービスプロパティタグ初期化
            m_clTagName = new EgSDBTagName(srMyName);
            m_clTagVal = new EgSDBTagVal();

        }

        /// <summary>
        /// デストラクタ
        /// </summary>
        ~SDBManeger()
        {
            // MMFの解放
            DisposeMmf();

            ExitEdge();
        }

        /// <summary>
        /// 指定したファイルからアイテム情報を取得する。 
        /// </summary>
        /// <param name="srFilePath">ファイルパス</param>
        /// <returns>アイテム情報</returns>
        private SDBItem GetSDBItem(string srFilePath)
        {
            // ファイル情報の生成
            FileInfo clFI = new FileInfo(srFilePath);

            // SDBItemの生成
            SDBItem clItem = new SDBItem();
            // ファイル名の設定
            clItem.FileName = clFI.Name;
            // ファイルサイズの設定
            clItem.FileSize = clFI.Length;
            // 更新日時の設定
            clItem.UpdateTime = clFI.LastWriteTime;

            return clItem;
        }

        /// <summary>
        /// Edgeメッセージハンドラ
        /// </summary>
        /// <param name="senderName">メッセージ送信元</param>
        /// <param name="messNo">メッセージ番号</param>
        /// <param name="paramArray">メッセージパラメータ</param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        private Int32 MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            Int32 inRet;                                    // 戻り値
            Byte byTagLen;                                  // タグレングス
            bool boFind;                                    // 検索結果(false=なし/true=あり)
            UInt16 woMsgPos;                                // レコード構成メッセージ参照位置
            bool boCheckOk;                                 // チェック結果(false=NG/true=OK)

            object obVal = new object();                    // タグの読み込み値
            Stopwatch sw = new Stopwatch();                 // 処理時間

            // インジケータサービスの実行状態更新
            m_clIndVal.m_dwLive++;
            m_EgApi.EgTagWrite(m_clIndName.Live, m_clIndVal.m_dwLive);

            // ========================================
            switch (messNo)
            {
                // ========================================
                // サービス開始指令
                case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:
                    // インジケータサービスの実行状態判定 ... 既に実行中の場合は更新しない
                    if (m_clIndVal.m_boRun != true)
                    {
                        // インジケータサービスの実行状態更新
                        m_clIndVal.m_boRun = true;
                        m_EgApi.EgTagWrite(m_clIndName.Run, m_clIndVal.m_boRun);
                        // コレクタ開始を送信
                        StartCollector();
                    }
                    break;

                // ========================================
                // サービス終了指令
                case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:
                    // インジケータサービスの実行状態判定 ... 既に停止中の場合は更新しない
                    if (m_clIndVal.m_boRun != false)
                    {
                        // インジケータサービスの実行状態更新
                        m_clIndVal.m_boRun = false;
                        m_EgApi.EgTagWrite(m_clIndName.Run, m_clIndVal.m_boRun);
                        // コレクタ停止を送信
                        StopCollector();
                    }
                    // フレームワークの終了
                    ExitEdge();
                    break;

                // ========================================
                // サービス一時停止指令
                case (int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE:
                    // インジケータサービスの実行状態判定 ... 既に停止中の場合は更新しない
                    if (m_clIndVal.m_boRun != false)
                    {
                        // インジケータサービスの実行状態更新
                        m_clIndVal.m_boRun = false;
                        m_EgApi.EgTagWrite(m_clIndName.Run, m_clIndVal.m_boRun);
                        // コレクタ停止を送信
                        StopCollector();
                    }
                    break;

                // ========================================
                // サービスアップデート指令
                case (int)EGDEFINE.EGMSG.EM_SERVICE_UPDATE:
                    // このコンテナでは何もしない
                    break;

                // ========================================
                // レコード構成追加
                case (int)SDBMsgNo.CreateFormat:
                    // 応答結果
                    Int32 inTagResultVal = 0;
                    // 属性値
                    object obType = new object();
                    object obSize = new object();
                    // レコード構成メッセージ参照位置をタグ数にセット
                    woMsgPos = 6;
                    // レコード構成の生成
                    SDBRecordFormat clFmt = new SDBRecordFormat();
                    // タグ情報の確保:　応答結果, 応答レコード構成番号は除く
                    clFmt.Init((Byte)(paramArray[woMsgPos] - 2));
                    // レコード構成番号をヘッダから取得
                    clFmt.Number = m_SDBList[m_SDBList.Count - 1].FileHeader.FormatNumber;
                    // レコード構成メッセージ参照位置をタグ[0].レングスにセット
                    woMsgPos++;
                    // 応答結果.タグレングスの取得
                    byTagLen = paramArray[woMsgPos];
                    // レコード構成メッセージ参照位置をタグ[i].タグ名にセット
                    woMsgPos++;
                    // 応答結果.タグ名のセット
                    String srTagResult = Encoding.GetEncoding("Shift_JIS").GetString(paramArray, woMsgPos, byTagLen);
                    // レコード構成メッセージ参照位置をタグ[i].レングスにセット
                    woMsgPos += byTagLen;
                    // 応答レコード構成番号.タグレングスの取得
                    byTagLen = paramArray[woMsgPos];
                    // レコード構成メッセージ参照位置をタグ[i].タグ名にセット
                    woMsgPos++;
                    // 応答結果.タグ名のセット
                    String srTagFormat = Encoding.GetEncoding("Shift_JIS").GetString(paramArray, woMsgPos, byTagLen);
                    // レコード構成メッセージ参照位置をタグ[i].レングスにセット
                    woMsgPos += byTagLen;
                    // タグ数ループ
                    for (var i = 0; i < clFmt.TagCount; i++)
                    {
                        // タグレングスの取得
                        byTagLen = paramArray[woMsgPos];
                        // レコード構成メッセージ参照位置をタグ[i].タグ名にセット
                        woMsgPos++;
                        // タグ名のセット
                        String srTagName = Encoding.GetEncoding("Shift_JIS").GetString(paramArray, woMsgPos, byTagLen);
                        // タグタイプの取得
                        GetTagProperty(srTagName, "Type", ref obType);
                        // タグサイズの取得
                        GetTagProperty(srTagName, "Size", ref obSize);
                        // タグ情報の追加
                        clFmt.AddTags((UInt16)i, srTagName, (byte)((UInt16)obType), (byte)((UInt16)obSize));
                        // レコード構成メッセージ参照位置をタグ[i].レングスにセット
                        woMsgPos += byTagLen;
                    }
                    // プロセス内で管理しているリストに既に登録されているかの判定
                    boFind = false;                                             // デフォルトはなし
                    foreach (SDBRecordFormat clRecord in m_Record)              // 登録レコード数ループ
                    {
                        // タグ数が違う場合は次のレコードへ
                        if (clRecord.TagCount != clFmt.TagCount) { continue; }

                        bool boTagMatch = true;                                 // タグ比較結果(false=不一致/true=一致)

                        // タグ名の比較
                        for (var i = 0; i < clRecord.Tags.Length; i++)          // タグ数ループ
                        {
                            // タグ名を比較
                            if (clRecord.Tags[i].GetName() != clFmt.Tags[i].GetName())
                            {
                                boTagMatch = false;                             // タグ不一致
                                break;                                          // タグ名の比較を終了
                            }
                        }

                        // タグ比較判定
                        if (true == boTagMatch)                                 // タグ一致
                        {
                            // 検索結果は登録状態をセット
                            // false   ... ファイル切替ありのため、レコード構成追加あり
                            // true    ... ファイル切替なしのため、レコード構成追加なし
                            boFind = clRecord.Configure;
                            clFmt.Number = clRecord.Number;                     // 一致したレコード構成番号をセット
                            inTagResultVal = (Int32)SDBErr.ResultOk;            // 応答結果成功に更新
                            break;                                              // 登録レコード数ループを終了
                        }
                    }
                    if (false == boFind)                                        // レコード未検出
                    {
                        // レコード構成の追加
                        inRet = AddRecordFormat(clFmt);
                        if (inRet == (Int32)SDBErr.Ok)                          // レコード構成追加成功
                        {
                            inTagResultVal = (Int32)SDBErr.ResultOk;            // 応答結果成功に更新
                        }
                        else
                        {
                            inTagResultVal = inRet;                             // 応答結果失敗をセット
                        }
                    }
                    // 応答結果を書き込み
                    m_EgApi.EgTagWrite(srTagResult, inTagResultVal);
                    // 応答レコード構成番号を書き込み
                    m_EgApi.EgTagWrite(srTagFormat, clFmt.Number);
                    break;

                // ========================================
                // レコードバイナリ追加
                case (int)SDBMsgNo.InsertBinary:

                    // ============================================================
                    // 処理時間の計測開始：
                    // - タグの計測回数が有効(1以上がセットされている)の場合は、処理時間を計測する。
                    // ============================================================
                    // 計測回数読み込み
                    m_EgApi.EgTagRead(m_clTagName.MeasureCount, ref obVal);
                    m_clTagVal.m_inMeasureCount = (int)obVal;
                    // 合計レコード追加時間読み込み
                    m_EgApi.EgTagRead(m_clTagName.TotalRecTime, ref obVal);
                    m_clTagVal.m_doTotalRecTime = (double)obVal;
                    // 処理時間計測開始判定：計測回数が1以上の場合は、処理時間計測開始
                    if (0 < m_clTagVal.m_inMeasureCount) { sw.Start(); }

                    // メッセージサイズ
                    UInt16 woMsgSize = 0;
                    // チェック結果のデフォルトはOKにセット
                    boCheckOk = true;
                    // レコードサイズチェック
                    // メッセージデータが 最低メッセージ番号 + メッセージサイズ + レコード構成番号 はあるか?
                    if (paramArray.Length < 10)
                    {
                        // 指定サイズ未満のため、NG
                        boCheckOk = false;
                    }
                    // メッセージサイズ判定
                    if (boCheckOk == true)
                    {
                        // レコードバイナリメッセージ参照位置をタグ数にセット
                        woMsgPos = 4;
                        // メッセージサイズの取得
                        woMsgSize = BitConverter.ToUInt16(paramArray, woMsgPos);
                        if (woMsgPos == 0)
                        {
                            // メッセージサイズ未設定のため、NG
                            boCheckOk = false;
                        }
                    }
                    // メッセージデータ判定
                    if (boCheckOk == true)
                    {
                        // メッセージデータが指定されたメッセージサイズ未満の場合はNG
                        if (paramArray.Length < (woMsgSize - 4))
                        {
                            // 指定サイズ未満のため、NG
                            boCheckOk = false;
                        }
                    }
                    // チェックOKならレコードバイナリの追加
                    if (boCheckOk == true)
                    {
                        // メッセージ番号を取り除いたバイト配列をレコードバイナリとして追加する。
                        AddRecord(paramArray, 4, woMsgSize);
                    }


                    // ============================================================
                    // 処理時間の更新：
                    // - タグの計測回数が有効(1以上がセットされている)の場合は、処理時間を更新する。
                    // ============================================================
                    // 処理時間更新判定：計測回数が1以上の場合は、処理時間を更新する。
                    if (0 < m_clTagVal.m_inMeasureCount)
                    {
                        sw.Stop();
                        // 合計レコード追加時間の加算し、タグを書き込む
                        m_clTagVal.m_doTotalRecTime += sw.Elapsed.TotalMilliseconds;
                        m_EgApi.EgTagWrite(m_clTagName.TotalRecTime, m_clTagVal.m_doTotalRecTime);
                        // 計測回数をインクリメントし、タグを書き込む
                        m_clTagVal.m_inMeasureCount--;
                        m_EgApi.EgTagWrite(m_clTagName.MeasureCount, m_clTagVal.m_inMeasureCount);
                    }
                    break;

                // ========================================
                // 未サポート
                default:
                    break;

            }

            return (Int32)SDBErr.Ok;
        }

        /// <summary>
        /// MMFを生成する。
        /// </summary>
        /// <param name="srFilePath"></param>
        /// <returns>0=成功/それ以外=エラー</returns>
        private Int32 CreateMmf(String srFilePath)
        {

            // ファイルのオープン
            // - OpenOrCreate ... ファイルがある場合はオープン、ファイルがない場合は新規作成
            m_SDBMmf = MemoryMappedFile.CreateFromFile(srFilePath, FileMode.OpenOrCreate, Path.GetFileName(srFilePath), FragmentSize);
            if (m_SDBMmf == null)
            {
                // ファイルのオープン失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: MemoryMappedFile.CreateFromFile");
                return (Int32)SDBErr.OpenFile;
            }

            // Viewの生成
            m_SDBView = m_SDBMmf.CreateViewStream();
            if (m_SDBView == null)
            {
                // アクセサの生成失敗のため、MMFを削除
                DisposeMmf();
                // アクセサの生成失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: MemoryMappedFile.CreateViewStream");
                return (Int32)SDBErr.OpenFile;
            }

            return (Int32)SDBErr.Ok;
        }

        /// <summary>
        /// MMFを削除する。
        /// </summary>
        /// <param name="srFilePath"></param>
        /// <returns>0=成功/それ以外=エラー</returns>
        private Int32 DisposeMmf()
        {

            if (null != m_SDBMmf)
            {
                m_SDBMmf.Dispose();
                m_SDBMmf = null;
            }

            if (null != m_SDBView)
            {
                m_SDBView.Dispose();
                m_SDBView = null;
            }

            return (Int32)SDBErr.Ok;
        }


        /// <summary>
        /// edgeフレームワーク初期化
        /// </summary>
        /// <returns></returns>
        public Int32 InitEdge()
        {
            // 戻り値
            Int32 inRet;

            // ========================================
            // Edgeフレームワーク動作設定構造体の設定
            edge_API.EDGECONFIG stConfig = new edge_API.EDGECONFIG();
            // 初期値設定
            stConfig = m_EgApi.EDGE_CONFIG_DEFAULT;
            //Edgeメッセージユーザハンドラの指定
            stConfig.mInstMessFunc = MsgHandler;

            // ========================================
            // Edgeフレームワークの初期化
            inRet = m_EgApi.EgInit(stConfig);
            // 結果判定
            if (0 != inRet)
            {
                // 初期化失敗
                m_clTagVal.m_inLastErr = inRet;
                return inRet;
            }
            if(m_EgApi.EGFW.ServiceRealName != null && m_EgApi.EGFW.ServiceRealName.Length > 0) //何かしら文字列がある場合格納
                ServiceRealName = m_EgApi.EGFW.ServiceRealName; //サービス名代入

            m_clIndName.ReSetEgSDBIndName(ServiceRealName);// サービスインジケータ 再設定
            m_clTagName.ReSetEgSDBTagName(ServiceRealName);// サービスプロパティタグ 再設定

            // サービスインジケータの読み込み
            inRet = fnReadIndVal();
            if (0 != inRet)
            {
                // 読み込み失敗
                m_clTagVal.m_inLastErr = inRet;
                return inRet;
            }

            // サービスプロパティタグの生成
            inRet = fnCreateTagVal();
            if (0 != inRet)
            {
                // 生成失敗
                m_clTagVal.m_inLastErr = inRet;
                return inRet;
            }

            // サービスプロパティタグの読み込み
            inRet = fnReadTagVal();
            if (0 != inRet)
            {
                // 読み込み失敗
                m_clTagVal.m_inLastErr = inRet;
                return inRet;
            }

            // コレクタ受信待ちスレッドの生成
            m_thCollectorTS = new CancellationTokenSource();
            // トークン指定
            CancellationToken clCollectorToken = m_thCollectorTS.Token;
            // スレッドの開始
            ThreadCollector(clCollectorToken);

            // インジケータサービスの起動状態更新
            m_clIndVal.m_byStatus = 1;
            m_EgApi.EgTagWrite(m_clIndName.Status, m_clIndVal.m_byStatus);

            return (Int32)SDBErr.Ok;
        }

        /// <summary>
        /// サービスインジケータの読み込み
        /// </summary>
        /// <returns></returns>
        private int fnReadIndVal()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値
            object obVal = new object();                    // タグの読み込み値
            string srDesc = "";                             // 処理内容

            // サービスインジケータの読み込み[.Status]
            srDesc = "TagRead " + m_clIndName.Status;
            inRet = m_EgApi.EgTagRead(m_clIndName.Status, ref obVal);
            if (inRet == 0)
            {
                // 読み込み成功
                // タグのデータ型取得
                m_clIndVal.m_byStatus = (byte)obVal;
            }
            else
            {
                // 読み込み失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                return (int)SDBErr.IndicatorNotConfig;
            }

            // サービスインジケータの読み込み[.ERROR]
            srDesc = "TagRead " + m_clIndName.Error;
            inRet = m_EgApi.EgTagRead(m_clIndName.Error, ref obVal);
            if (inRet == 0)
            {
                // 読み込み成功
                // タグのデータ型取得
                m_clIndVal.m_boError = (bool)obVal;
            }
            else
            {
                // 読み込み失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                return (int)SDBErr.IndicatorNotConfig;
            }

            // サービスインジケータの読み込み[.RUN]
            srDesc = "TagRead " + m_clIndName.Run;
            inRet = m_EgApi.EgTagRead(m_clIndName.Run, ref obVal);
            if (inRet == 0)
            {
                // 読み込み成功
                // タグのデータ型取得
                m_clIndVal.m_boRun = (bool)obVal;
            }
            else
            {
                // 読み込み失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                return (int)SDBErr.IndicatorNotConfig;
            }

            // サービスインジケータの読み込み[.LIVE]
            srDesc = "TagRead " + m_clIndName.Live;
            inRet = m_EgApi.EgTagRead(m_clIndName.Live, ref obVal);
            if (inRet == 0)
            {
                // 読み込み成功
                // タグのデータ型取得
                m_clIndVal.m_dwLive = (UInt32)obVal;
            }
            else
            {
                // 読み込み失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                return (int)SDBErr.IndicatorNotConfig;
            }

            return inRet;
        }

        /// <summary>
        /// サービスプロパティタグの生成
        /// </summary>
        /// <returns></returns>
        private int fnCreateTagVal()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値
            object obVal = new object();                    // タグの読み込み値
            string srDesc = "";                             // 処理内容

            try
            {
                // サービスインジケータの生成[.LastErr]
                srDesc = "TagCreate " + m_clTagName.LastErr;
                inRet = m_EgApi.EgTagCreateEx(m_clTagName.LastErr, (Int32)EGDEFINE.egTagDataType.Int32, "", "");
                if (inRet != 0)
                {
                    // 生成失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    return (int)SDBErr.TagCreate;
                }
                // タグの書き込み ... 最新のエラーコード
                inRet = m_EgApi.EgTagWrite(m_clTagName.LastErr, m_clTagVal.m_inLastErr);

                // サービスインジケータの生成[.FileFreeSize]
                srDesc = "TagCreate " + m_clTagName.FileFreeSize;
                inRet = m_EgApi.EgTagCreateEx(m_clTagName.FileFreeSize, (Int32)EGDEFINE.egTagDataType.Int64, "", "");
                if (inRet != 0)
                {
                    // 生成失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    return (int)SDBErr.TagCreate;
                }

                // サービスインジケータの生成[.FormatNumber]
                srDesc = "TagCreate " + m_clTagName.FormatNumber;
                inRet = m_EgApi.EgTagCreateEx(m_clTagName.FormatNumber, (Int32)EGDEFINE.egTagDataType.Int32, "", "");
                if (inRet != 0)
                {
                    // 生成失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    return (int)SDBErr.TagCreate;
                }

                // サービスインジケータの生成[.AddPointer]
                srDesc = "TagCreate " + m_clTagName.AddPointer;
                inRet = m_EgApi.EgTagCreateEx(m_clTagName.AddPointer, (Int32)EGDEFINE.egTagDataType.UInt32, "", "");
                if (inRet != 0)
                {
                    // 生成失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    return (int)SDBErr.TagCreate;
                }

                // サービスインジケータの生成[.FileCount]
                srDesc = "TagCreate " + m_clTagName.FileCount;
                inRet = m_EgApi.EgTagCreateEx(m_clTagName.FileCount, (Int32)EGDEFINE.egTagDataType.Int32, "", "");
                if (inRet != 0)
                {
                    // 生成失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    return (int)SDBErr.TagCreate;
                }

                // サービスインジケータの生成[.CapacityFreeSize]
                srDesc = "TagCreate " + m_clTagName.CapacityFreeSize;
                inRet = m_EgApi.EgTagCreateEx(m_clTagName.CapacityFreeSize, (Int32)EGDEFINE.egTagDataType.Int64, "", "");
                if (inRet != 0)
                {
                    // 生成失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    return (int)SDBErr.TagCreate;
                }

                // サービスプロパティタグの生成[.MeasureCount]
                srDesc = "TagCreate " + m_clTagName.MeasureCount;
                inRet = m_EgApi.EgTagCreateEx(m_clTagName.MeasureCount, (Int32)EGDEFINE.egTagDataType.Int32, "", "");
                if (inRet != 0)
                {
                    // 生成失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    return (int)SDBErr.TagCreate;
                }

                // サービスプロパティタグの生成[.TotalRecTime]
                srDesc = "TagCreate " + m_clTagName.TotalRecTime;
                inRet = m_EgApi.EgTagCreateEx(m_clTagName.TotalRecTime, (Int32)EGDEFINE.egTagDataType.Double, "", "");
                if (inRet != 0)
                {
                    // 生成失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    return (int)SDBErr.TagCreate;
                }


            }
            catch (Exception ex)
            {
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Failed: " + srDesc + " 0x" + ex.HResult.ToString("X8"));
            }

            return inRet;
        }

        /// <summary>
        /// サービスプロパティタグの読み込み
        /// </summary>
        /// <returns></returns>
        private int fnReadTagVal()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値

            // サービスインジケータの読み込み[.AutoRun]
            fnReadTagVal_AutoRun();

            // サービスインジケータの読み込み[.FragmentSize]
            fnReadTagVal_FragmentSize();

            // サービスインジケータの読み込み[.CapacitySize]
            fnReadTagVal_CapacitySize();

            // サービスインジケータの読み込み[.StorePath]
            fnReadTagVal_StorePath();

            return inRet;
        }

        /// <summary>
        /// サービスプロパティタグ[.AutoRun]の読み込み
        /// </summary>
        /// <returns></returns>
        private int fnReadTagVal_AutoRun()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値
            object obVal = new object();                    // タグの読み込み値
            string srDesc = "";                             // 処理内容

            // サービスインジケータの読み込み[.AutoRun]
            srDesc = "TagRead " + m_clTagName.AutoRun;
            inRet = m_EgApi.EgTagRead(m_clTagName.AutoRun, ref obVal);
            if (inRet == 0)
            {
                // 読み込み成功
                m_clTagVal.m_boAutoRun = (bool)obVal;
            }
            else
            {
                // 読み込み失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                // タグの生成
                m_EgApi.EgTagCreateEx(m_clTagName.AutoRun, (ushort)EGDEFINE.egTagDataType.Boolean, "", "EgSDAT AutoRun");
                // 初期値をセット
                m_clTagVal.m_boAutoRun = true;
                // タグの書き込み ... 自動起動設定
                m_EgApi.EgTagWrite(m_clTagName.AutoRun, m_clTagVal.m_boAutoRun);
            }

            return inRet;
        }

        /// <summary>
        /// サービスプロパティタグ[.FragmentSize]の読み込み
        /// </summary>
        /// <returns></returns>
        private int fnReadTagVal_FragmentSize()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値
            object obVal = new object();                    // タグの読み込み値
            string srDesc = "";                             // 処理内容

            // サービスインジケータの読み込み[.FragmentSize]
            srDesc = "TagRead " + m_clTagName.FragmentSize;
            inRet = m_EgApi.EgTagRead(m_clTagName.FragmentSize, ref obVal);
            if (inRet == 0)
            {
                // 読み込み成功
                FragmentSize = (Int64)obVal;
            }
            else
            {
                // 読み込み失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                // タグの生成
                m_EgApi.EgTagCreateEx(m_clTagName.FragmentSize, (ushort)EGDEFINE.egTagDataType.Int64, "", "EgSDAT FragmentSize");
                // 初期値をセット
                FragmentSize = SDBCONST.FRAGMENT_SIZE;
                // タグの書き込み ... ファイル単位サイズ
                m_EgApi.EgTagWrite(m_clTagName.FragmentSize, FragmentSize);
            }

            // 最小値の算出
            Int64 loMinVal = SDBCONST.MIN_FRAGMENT_SIZE;

            // 読み込んだ値の範囲チェック
            if (loMinVal > FragmentSize)
            {
                // 指定した最小値未満のため、指定した最小値に合わせる
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Change value: " + m_clTagName.FragmentSize + " " + FragmentSize.ToString() + " -> " + loMinVal);
                // 最小値をセット
                FragmentSize = loMinVal;
                // タグの書き込み ... ファイル単位サイズ
                m_EgApi.EgTagWrite(m_clTagName.FragmentSize, FragmentSize);
            }

            return inRet;
        }

        /// <summary>
        /// サービスプロパティタグ[.CapacitySize]の読み込み
        /// </summary>
        /// <returns></returns>
        private int fnReadTagVal_CapacitySize()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値
            object obVal = new object();                    // タグの読み込み値
            string srDesc = "";                             // 処理内容

            // サービスインジケータの読み込み[.CapacitySize]
            srDesc = "TagRead " + m_clTagName.CapacitySize;
            inRet = m_EgApi.EgTagRead(m_clTagName.CapacitySize, ref obVal);
            if (inRet == 0)
            {
                // 読み込み成功
                CapacitySize = (Int64)obVal;
            }
            else
            {
                // 読み込み失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                // タグの生成
                m_EgApi.EgTagCreateEx(m_clTagName.CapacitySize, (ushort)EGDEFINE.egTagDataType.Int64, "", "EgSDAT CapacitySize");
                // 初期値をセット
                CapacitySize = SDBCONST.CAPACITY_SIZE;
                // タグの書き込み ... 記録容量
                m_EgApi.EgTagWrite(m_clTagName.CapacitySize, CapacitySize);
            }

            // 最小値の算出
            Int64 loMinVal = FragmentSize * SDBCONST.MIN_CAPACITY_RATE;

            // 読み込んだ値の範囲チェック
            if (loMinVal > CapacitySize)
            {
                // 指定した最小値未満のため、指定した最小値に合わせる
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Change value: " + m_clTagName.CapacitySize + " " + CapacitySize.ToString() + " -> " + loMinVal);
                // 最小値をセット
                CapacitySize = loMinVal;
                // タグの書き込み ... 記録容量
                m_EgApi.EgTagWrite(m_clTagName.CapacitySize, CapacitySize);
            }

            return inRet;
        }

        /// <summary>
        /// サービスプロパティタグ[.StorePath]の読み込み
        /// </summary>
        /// <returns></returns>
        private int fnReadTagVal_StorePath()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値
            object obVal = new object();                    // タグの読み込み値
            string srDesc = "";                             // 処理内容

            // サービスインジケータの読み込み[.StorePath]
            srDesc = "TagRead " + m_clTagName.StorePath;
            inRet = m_EgApi.EgTagRead(m_clTagName.StorePath, ref obVal);
            if (inRet == 0)
            {
                srDesc = "TagGetProperty.Address " + m_clTagName.StorePath;
                // 読み込み成功 ... タグ属性のアドレスを取得
                inRet = m_EgApi.EgTagGetProperty(m_clTagName.StorePath, "Address", ref obVal);
                if (inRet == 0)
                {
                    // 取得成功
                    StorePath = (string)obVal;
                }
                else
                {
                    // 取得失敗
                    edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                    // デフォルト値をセット
                    StorePath = Directory.GetCurrentDirectory();
                }
            }
            else
            {
                // 読み込み失敗
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Failed: " + srDesc + " 0x" + inRet.ToString("X8"));
                // デフォルト値をセット
                StorePath = Directory.GetCurrentDirectory();
            }
            edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_DBG, "StorePath: " + StorePath);

            return inRet;
        }

        /// <summary>
        /// Edgeフレームワークの終了
        /// </summary>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public Int32 ExitEdge()
        {

            // コレクタ受信待ちスレッドの終了
            if (m_thCollectorTS != null)
            {
                //キャンセル状態をセットし、スレッド処理で抜ける
                m_thCollectorTS.Cancel();
                // 変数の初期化
                m_thCollectorTS = null;
            }

            // インジケータサービスの起動状態更新
            m_clIndVal.m_byStatus = 0;
            m_EgApi.EgTagWrite(m_clIndName.Status, m_clIndVal.m_byStatus);

            // SDBファイルのクローズ
            CloseFile(CloseFileFactor.ExitContainer);

            // ========================================
            // Edgeフレームワークの終了
            m_EgApi.EgFinalize();
            
            // アプリケーションの終了
            Environment.Exit(0);

            return (Int32)SDBErr.Ok;
        }


        /// <summary>
        /// タグの属性を取得する
        /// </summary>
        /// <param name="srName">タグ名</param>
        /// <returns>マイナス値=取得失敗</returns>
        public Int32 GetTagProperty(string srTagName, string srPropName, ref object obVal)
        {
            return m_EgApi.EgTagGetProperty(srTagName, srPropName, ref obVal);
        }


        /// <summary>
        /// 指定フォルダからSDBファイルリストを作成する。
        /// </summary>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        /// <remarks>ファイルリストは更新日時昇順で収納されます。</remarks>
        public Int32 CreateList()
        {

            // SDBファイルリストのクリア
            m_SDBList.Clear();

            // レコード構成リストのクリア
            m_Record.Clear();

            // 検索パターンの設定
            String srPattern = "*_" + SDBCONST.FILE_SUFFIX + "." + SDBCONST.FILE_EXT;

            // SDBファイル収納パスからSDBファイルを更新日時昇順で検索
            IEnumerable<string> srFiles = Directory.EnumerateFiles(StorePath, srPattern)
                                                    .OrderBy(f => File.GetLastWriteTime(f));

            foreach (string srFile in srFiles)
            {
                // SDBItemに追加
                m_SDBList.Add(GetSDBItem(srFile));
            }

            return (Int32)SDBErr.Ok;
        }

            /// <summary>
            /// 取得したリストから残り容量を返す。
            /// </summary>
            /// <returns>残り容量</returns>
            public Int64 GetCapacityFreeSize()
        {

            // 使用容量
            Int64 loUseSize = 0;

            // SDBファイルリストループ
            foreach (var clItem in m_SDBList)
            {
                // ファイルサイズを使用容量に加算
                loUseSize += clItem.FileSize;
            }

            // 残り容量の計算：残り容量 = 記録容量 - 使用容量
            Int64 loCapacityFreeSize = (CapacitySize - loUseSize);
            // タグの書き込み ... 残り記録容量
            m_EgApi.EgTagWrite(m_clTagName.CapacityFreeSize, loCapacityFreeSize);
            return loCapacityFreeSize;
        }

        /// <summary>
        /// ファイル上限と書き込み位置からファイル内の残り容量を求める。
        /// </summary>
        /// <returns>残り容量</returns>
        public Int64 GetFileFreeSize()
        {
            // 残り容量の計算：残り容量 = 記録容量 - 使用容量
            Int64 loFileFreeSize = FragmentSize - (Int64)m_SDBList[m_SDBList.Count - 1].FileHeader.AddPointer;
            // タグの書き込み ... 残りファイルサイズ
            m_EgApi.EgTagWrite(m_clTagName.FileFreeSize, loFileFreeSize);
            return loFileFreeSize;
        }

        /// <summary>
        /// 使用するファイル名を取得する
        /// </summary>
        /// <returns></returns>
        public String GetUseFileName()
        {
            // ファイル名
            string srFileName;

            // ファイルリストの判定
            if (m_SDBList.Count > 0)
            {
                // ファイルリストあり
                srFileName = m_SDBList[m_SDBList.Count - 1].FileName;
            }
            else
            {
                // ファイルリストなし
                srFileName = GetNextFileName();
            }

            return srFileName;
        }

        /// <summary>
        /// 次のファイル名を求める。
        /// </summary>
        /// <returns>次のファイル名</returns>
        public String GetNextFileName()
        {
            // ファイル名の設定："現在日時_サフィックス.拡張子" 
            return (DateTime.Now.ToString("yyyyMMddHHmmss") + "_" + SDBCONST.FILE_SUFFIX + "." + SDBCONST.FILE_EXT);
        }

        /// <summary>
        /// ファイルヘッダ.最新のレコード構成番号の更新
        /// </summary>
        private void AddHeaderFormatNumber()
        {
            // 最新のレコード構成番号の更新
            AddHeaderFormatNumber(m_SDBList[m_SDBList.Count - 1].m_stFileHeader.FormatNumber + 1);
        }

        /// <summary>
        /// ファイルヘッダ.最新のレコード構成番号の更新
        /// </summary>
        /// <param name="inFormatNumber">レコード構成番号</param>
        private void AddHeaderFormatNumber(int inFormatNumber)
        {
            // 最新のレコード構成番号の更新
            m_SDBList[m_SDBList.Count - 1].m_stFileHeader.FormatNumber = inFormatNumber;
            // タグの書き込み ... 最新のレコード構成番号
            m_EgApi.EgTagWrite(m_clTagName.FormatNumber, m_SDBList[m_SDBList.Count - 1].m_stFileHeader.FormatNumber);
            // 設定位置の変更
            m_SDBView.Position = FileHeader.PosFormatNumber;
            // SDBファイルの更新
            m_SDBView.WriteAsync(BitConverter.GetBytes(m_SDBList[m_SDBList.Count - 1].m_stFileHeader.FormatNumber),
                                    0,
                                    Marshal.SizeOf(m_SDBList[m_SDBList.Count - 1].m_stFileHeader.FormatNumber));
        }

        /// <summary>
        /// ファイルヘッダ.ヘッダサイズの更新
        /// </summary>
        /// <param name="bySize">ヘッダサイズ</param>
        private void AddHeaderSize(Byte bySize)
        {
            // ヘッダサイズの更新
            m_SDBList[m_SDBList.Count - 1].m_stFileHeader.HeadSize += bySize;
            // 設定位置の変更
            m_SDBView.Position = FileHeader.PosHeadSize;
            // SDBファイルの更新
            m_SDBView.WriteAsync(BitConverter.GetBytes(m_SDBList[m_SDBList.Count - 1].m_stFileHeader.HeadSize),
                                    0,
                                    Marshal.SizeOf(m_SDBList[m_SDBList.Count - 1].m_stFileHeader.HeadSize));
        }

        /// <summary>
        /// ファイルヘッダ.書き込み位置の更新
        /// </summary>
        /// <param name="dwAddPointer">書き込み位置</param>
        private void AddHeaderPointer(UInt32 dwAddPointer)
        {
            // 書き込み位置の更新
            m_SDBList[m_SDBList.Count - 1].m_stFileHeader.AddPointer += dwAddPointer;
            m_EgApi.EgTagWrite(m_clTagName.AddPointer, m_SDBList[m_SDBList.Count - 1].m_stFileHeader.AddPointer);
            // 設定位置の変更
            m_SDBView.Position = FileHeader.PosAddPointer;
            // SDBファイルの更新
            m_SDBView.WriteAsync(BitConverter.GetBytes(m_SDBList[m_SDBList.Count - 1].m_stFileHeader.AddPointer),
                                    0,
                                    Marshal.SizeOf(m_SDBList[m_SDBList.Count - 1].m_stFileHeader.AddPointer));
        }

        /// <summary>
        /// ファイルヘッダの書き込み。
        /// </summary>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public Int32 WriteHeader()
        {

            // ファイルヘッダの初期化
            m_SDBList[m_SDBList.Count - 1].m_stFileHeader.Init();

            // ファイルヘッダをバイト配列に変換
            byte[] byHeader = new byte[SDBCONST.FILEHEADER_SIZE];
            // アンマネージメモリからメモリを割り当て
            IntPtr ptr = Marshal.AllocHGlobal(SDBCONST.FILEHEADER_SIZE);
            // マネージオブジェクトからアンマネージメモリにデータをマーシャリング
            Marshal.StructureToPtr(m_SDBList[m_SDBList.Count - 1].m_stFileHeader, ptr, false);
            // バイト配列にコピー
            Marshal.Copy(ptr, byHeader, 0, SDBCONST.FILEHEADER_SIZE);
            // SDBファイル更新:ファイルヘッダ
            m_SDBView.Position = 0;
            m_SDBView.WriteAsync(byHeader, 0, SDBCONST.FILEHEADER_SIZE);
            // メモリの解放
            Marshal.FreeHGlobal(ptr);

            // ファイルヘッダ.書き込み位置の更新
            AddHeaderPointer(m_SDBList[m_SDBList.Count - 1].m_stFileHeader.HeadSize);

            return (Int32)SDBErr.Ok;
        }

        /// <summary>
        /// ファイルヘッダの読み込み。
        /// </summary>
        /// <param name="inFileNumber">ファイル番号</param>
        /// <returns>ファイルヘッダ：</returns>
        public FileHeader ReadHeader(Int32 inFileNumber)
        {

            // ファイル番号判定
            if ((inFileNumber < 0) || (inFileNumber >= m_SDBList.Count))
            {
                // 範囲外
                FileHeader stDummy = new FileHeader();
                return stDummy;
            }

            return m_SDBList[inFileNumber].FileHeader;
        }

        // テスト用に構造体を
        public struct RecFmt
        {
            public byte[] m_byData;

        }

        /// <summary>
        /// ファイル切替処理
        /// </summary>
        /// <param name="loSize">サイズ</param>
        private void ChangeFile(Int64 loSize)
        {
            // サイズとファイルの残り容量の比較
            if (loSize > GetFileFreeSize())
            {
                // レコード構成サイズ > ファイルの残り容量
                // レコード構成番号の取得
                int inFormatNumber = m_SDBList[m_SDBList.Count - 1].m_stFileHeader.FormatNumber;
                // ファイルを閉じる
                CloseFile(m_SDBList.Count - 1, CloseFileFactor.ChangeFile);
                // 次のファイルのオープン
                OpenFile(Path.Combine(StorePath, GetNextFileName()));
                // ファイルヘッダ.最新のレコード構成番号の更新
                AddHeaderFormatNumber(inFormatNumber);
            }
        }

        /// <summary>
        /// 新しいレコードフォーマットを追加する。
        /// </summary>
        /// <param name="clRecordFormat">レコードフォーマット</param>
        /// <param name="boAdd">true = レコードフォーマットリストの追加あり</param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public Int32 AddRecordFormat(SDBRecordFormat clRecordFormat, bool boAdd = true)
        {
            string srDesc = "";                             // 処理内容

            // ファイル切替処理
            ChangeFile((Int32)clRecordFormat.Size);

            // レコードフォーマット情報をログに出力する
            srDesc = "AddRecordFormat: ";
            srDesc += "Size=" + clRecordFormat.Size.ToString() + " ";
            srDesc += "Number=" + clRecordFormat.Number.ToString() + " ";
            srDesc += "TagCount=" + clRecordFormat.TagCount.ToString() + " ";
            edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_DBG, srDesc);

            // SDBファイル更新:レコード構成.レコードサイズ
            m_SDBView.Position = m_SDBList[m_SDBList.Count - 1].FileHeader.AddPointer;
            m_SDBView.WriteAsync(BitConverter.GetBytes(clRecordFormat.Size), 0, Marshal.SizeOf(clRecordFormat.Size));
            AddHeaderPointer((UInt16)Marshal.SizeOf(clRecordFormat.Size));

            // SDBファイル更新:レコード構成.レコード構成番号
            m_SDBView.Position = m_SDBList[m_SDBList.Count - 1].FileHeader.AddPointer;
            m_SDBView.WriteAsync(BitConverter.GetBytes(clRecordFormat.Number), 0, Marshal.SizeOf(clRecordFormat.Number));
            AddHeaderPointer((UInt16)Marshal.SizeOf(clRecordFormat.Number));

            // SDBファイル更新:レコード構成.タグ数
            m_SDBView.Position = m_SDBList[m_SDBList.Count - 1].FileHeader.AddPointer;
            m_SDBView.WriteAsync(BitConverter.GetBytes(clRecordFormat.TagCount), 0, Marshal.SizeOf(clRecordFormat.TagCount));
            AddHeaderPointer((UInt16)Marshal.SizeOf(clRecordFormat.TagCount));

            // タグ情報
            for (var i = 0; i < clRecordFormat.TagCount; i++)
            {
                // レコードフォーマット情報をログに出力する
                srDesc = "Tag[" + i.ToString() + "]: ";
                srDesc += "Name=" + Encoding.GetEncoding("Shift_JIS").GetString(clRecordFormat.Tags[i].Name.Val, 0, clRecordFormat.Tags[i].Name.Size) + " ";
                srDesc += "Type=" + clRecordFormat.Tags[i].Type.ToString() + " ";
                srDesc += "Size=" + clRecordFormat.Tags[i].Size.ToString() + " ";
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_DBG, srDesc);

                // SDBファイル更新:レコード構成.タグ名
                m_SDBView.Position = m_SDBList[m_SDBList.Count - 1].FileHeader.AddPointer;
                m_SDBView.WriteAsync(clRecordFormat.Tags[i].Name.GetBytes(), 0, clRecordFormat.Tags[i].Name.GetLength());
                AddHeaderPointer((UInt16)clRecordFormat.Tags[i].Name.GetLength());

                // SDBファイル更新:レコード構成.タグタイプ
                m_SDBView.Position = m_SDBList[m_SDBList.Count - 1].FileHeader.AddPointer;
                m_SDBView.WriteAsync(BitConverter.GetBytes(clRecordFormat.Tags[i].Type), 0, Marshal.SizeOf(clRecordFormat.Tags[i].Type));
                AddHeaderPointer((UInt16)Marshal.SizeOf(clRecordFormat.Tags[i].Type));

                // SDBファイル更新:レコード構成.タグサイズ
                m_SDBView.Position = m_SDBList[m_SDBList.Count - 1].FileHeader.AddPointer;
                m_SDBView.WriteAsync(BitConverter.GetBytes(clRecordFormat.Tags[i].Size), 0, Marshal.SizeOf(clRecordFormat.Tags[i].Size));
                AddHeaderPointer((UInt16)Marshal.SizeOf(clRecordFormat.Tags[i].Size));

            }

            // レコードフォーマットリストの追加判定
            if (true == boAdd)
            {
                // レコード状態を登録済みに更新
                clRecordFormat.Configure = true;

                // レコードフォーマットリストの追加
                m_Record.Add(clRecordFormat);

                // ファイルヘッダ.最新のレコード構成番号の更新
                AddHeaderFormatNumber();
            }

            return (Int32)SDBErr.Ok;
        }

        /// <summary>
        /// レコードバイナリを追加する。
        /// </summary>
        /// <param name="byRecordBinary">レコードバイナリ</param>
        /// <param name="inRecordOffset">レコードバイナリオフセット</param>
        /// <param name="inRecordLength">レコードバイナリレングス</param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public Int32 AddRecord(Byte[] byRecordBinary, int inRecordOffset, int inRecordLength)
        {
            int i;                                          // ループインデックス

            // ファイル切替処理
            ChangeFile(inRecordLength);

            // 登録判定
            for (i = 0; i < m_Record.Count; i++)
            {
                // レコード構成番号取得
                int inFormatNumber = BitConverter.ToInt32(byRecordBinary, inRecordOffset + 2);

                // 違うレコード構成番号はスキップ
                if (inFormatNumber != m_Record[i].Number) { continue; }

                // 登録済みはスキップ
                if (true == m_Record[i].Configure) { continue; }

                // レコードフォーマット追加
                // この時はレコードフォーマットリストは既に存在しているため追加なし
                AddRecordFormat(m_Record[i], false);

                // レコード状態を登録済みに更新
                m_Record[i].Configure = true;
            }

            // SDBファイル更新:レコードバイナリ
            m_SDBView.Position = m_SDBList[m_SDBList.Count - 1].FileHeader.AddPointer;
            m_SDBView.WriteAsync(byRecordBinary, inRecordOffset, inRecordLength);
            AddHeaderPointer((UInt16)inRecordLength);

            return (Int32)SDBErr.Ok;
        }

        /// <summary>
        /// SDBファイルを開く。
        /// </summary>
        /// <returns>成功=ファイル番号(0..)/失敗=マイナス値</returns>
        public Int32 OpenFile()
        {
            return OpenFile(Path.Combine(StorePath, GetUseFileName()));
        }

        /// <summary>
        /// SDBファイルを開く。
        /// </summary>
        /// <param name="srFilePath">ファイルパス</param>
        /// <returns>成功=ファイル番号(0..)/失敗=マイナス値</returns>
        public Int32 OpenFile(string srFilePath)
        {
            // 戻り値
            Int32 inRet = (Int32)SDBErr.Ok;

            // MMFの生成
            inRet = CreateMmf(srFilePath);
            if (inRet != (Int32)SDBErr.Ok)
            {
                // ファイルオープン失敗
                return inRet;
            }

            edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_DBG, "OpenFile: " + Path.GetFileName(srFilePath));

            // アイテム情報を取得
            SDBItem clItem = GetSDBItem(srFilePath);

            // デフォルトは false(SDBファイルリストにアイテムなし)
            bool boFind = false;

            // SDBファイルリストからアイテム情報を検索
            for (var i = 0; i < m_SDBList.Count; i++)
            {
                // 検索条件：ファイル名が一致しているか?
                if (m_SDBList[i].FileName == clItem.FileName)
                {
                    // ファイル名が一致
                    boFind = true;
                    // ファイル番号のセット
                    inRet = i;
                    // 検索終了
                    break;
                }
            }

            // 取得したアイテム情報がSDBファイルリストになければ追加する。
            if (false == boFind)
            {
                // SDBファイルリストにアイテムを追加
                m_SDBList.Add(clItem);
                // ファイルヘッダの書き込み
                WriteHeader();
                // ファイル番号のセット
                inRet = (m_SDBList.Count - 1);
            }

            // ファイルヘッダをバイト配列に変換
            byte[] byHeader = new byte[SDBCONST.FILEHEADER_SIZE];
            // アンマネージメモリからメモリを割り当て
            IntPtr ptr = Marshal.AllocHGlobal(SDBCONST.FILEHEADER_SIZE);
            // マネージオブジェクトからアンマネージメモリにデータをマーシャリング
            Marshal.StructureToPtr(m_SDBList[inRet].m_stFileHeader, ptr, false);
            // ヘッダ情報の読み込み
            m_SDBView.Position = 0;
            m_SDBView.Read(byHeader, 0, SDBCONST.FILEHEADER_SIZE);
            // バイト配列から作業領域にコピー
            Marshal.Copy(byHeader, 0, ptr, SDBCONST.FILEHEADER_SIZE);
            // 作業領域から構造体へコピー
            m_SDBList[inRet].m_stFileHeader = (FileHeader)Marshal.PtrToStructure(ptr, m_SDBList[inRet].m_stFileHeader.GetType());
            // メモリの解放
            Marshal.FreeHGlobal(ptr);

            // タグの書き込み ... 残り容量
            GetCapacityFreeSize();                          // このAPI内で更新される

            // タグの書き込み ... 残りファイルサイズ
            GetFileFreeSize();                              // このAPI内で更新される

            // タグの書き込み ... ファイルの書き込み位置
            m_EgApi.EgTagWrite(m_clTagName.AddPointer, m_SDBList[inRet].m_stFileHeader.AddPointer);

            // タグの書き込み ... 最新のレコード構成番号
            m_EgApi.EgTagWrite(m_clTagName.FormatNumber, m_SDBList[inRet].m_stFileHeader.FormatNumber);

            // タグの書き込み ... ファイルの個数
            m_EgApi.EgTagWrite(m_clTagName.FileCount, m_SDBList.Count);

            // ファイル番号を返却
            return inRet;
        }

        /// <summary>
        /// SDBファイルを閉じる
        /// </summary>
        /// <param name="enFactor">ファイルクローズ要因</param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public Int32 CloseFile(CloseFileFactor enFactor)
        {
            // 戻り値
            Int32 inRet = (Int32)SDBErr.Ok;

            // リストの判定
            if (m_SDBList.Count > 0)
            {
                CloseFile(m_SDBList.Count - 1, enFactor);
            }

            return inRet;
        }

        /// <summary>
        /// SDBファイルを閉じる
        /// </summary>
        /// <param name="inFileNumber">ファイル番号</param>
        /// <param name="enFactor">ファイルクローズ要因</param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public Int32 CloseFile(Int32 inFileNumber, CloseFileFactor enFactor)
        {
            int i;                                          // ループインデックス

            // ファイル番号判定
            if ((inFileNumber < 0) || (inFileNumber >= m_SDBList.Count))
            {
                // 範囲外
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_ERR, "CloseFile: out of range. FileNumber=" + inFileNumber.ToString() + " ListCount=" + m_SDBList.Count.ToString());
                return -1;
            }

            edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_DBG, "CloseFile: " + m_SDBList[inFileNumber].FileName);

            // MMFの解放
            DisposeMmf();

            // 新規オープン後、レコード構成を書き込むため、レコード未登録に更新
            for (i = 0; i < m_Record.Count; i++)
            {
                m_Record[i].Configure = false;
            }

            // ファイルの削除
            DeleteFile(enFactor);

            return (Int32)SDBErr.Ok;
        }

        /// <summary>
        /// SDBファイルを削除する。
        /// </summary>
        /// <param name="enFactor">ファイルクローズ要因</param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public Int32 DeleteFile(CloseFileFactor enFactor)
        {
            // ファイルの残り容量
            long loCapacityFreeSize = 0;

            // ファイル削除判定結果 true=削除実行 false=削除停止
            bool boIsDelete = false;

            // ファイルクローズ要因によってファイル削除条件が異なります。
            //  ファイルクローズ要因      ファイル削除条件
            // ------------------------------------------------------------
            //  ファイル切替              フォルダの残り容量 >  0 になるまでファイルを削除する
            //  コンテナ終了              フォルダの残り容量 >= 0 になるまでファイルを削除する
            while (true)
            {
                // ファイルの残り容量の取得
                loCapacityFreeSize = GetCapacityFreeSize();

                // ファイル削除判定
                // ファイル削除判定結果のデフォルトは削除停止にセット
                boIsDelete = false;
                // ファイルクローズ要因で分岐
                switch (enFactor)
                {
                    case CloseFileFactor.ChangeFile:        // ファイル切替で発生した
                        if (loCapacityFreeSize >  0) { boIsDelete = false; }    // フォルダの残り容量 >  0 であるため削除停止
                        else                         { boIsDelete = true;  }    // フォルダの残り容量 >  0 でないため削除実行
                        break;
                    case CloseFileFactor.ExitContainer:     // コンテナ終了で発生した
                        if (loCapacityFreeSize >= 0) { boIsDelete = false; }    // フォルダの残り容量 >= 0 であるため削除停止
                        else                         { boIsDelete = true;  }    // フォルダの残り容量 >= 0 でないため削除実行
                        break;
                }
                // 削除停止の場合はループを終了する
                if (!boIsDelete) { break; }

                // 最古のファイルを削除
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_DBG, "DeleteFile: " + m_SDBList[0].FileName);
                File.Delete(Path.Combine(StorePath, m_SDBList[0].FileName));

                // SDBファイルリストを削除する。
                m_SDBList.RemoveAt(0);

                // タグの書き込み ... ファイルの個数
                m_EgApi.EgTagWrite(m_clTagName.FileCount, m_SDBList.Count);

            }

            return (Int32)SDBErr.Ok;
        }

        /// <summary>
        /// コレクタ受信待ちスレッド（セマフォからのキックあり）
        /// </summary>
        private async void ThreadCollector(CancellationToken cancelToken)
        {
            int inRet = 0;
            string srDesc = "";                             // 処理内容

            try
            {
                await Task.Run(() =>
                {
                    // コレクタの登録チェック
                    if (m_EgApi.EGFW.clctRefList.Count == 0)
                    {
                        // コレクタないため終了
                        edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Warning: Collector is none. ");
                        return;
                    }

                    // 受信カウント
                    UInt32 dwCount = 0;

                    while (true)
                    {
                        srDesc = "Check Token IsCancellationRequested";

                        // キャンセルトークンが来たら終了
                        if (cancelToken.IsCancellationRequested)
                        {
                            break;
                        }

                        //タスク処理を開始する
                        srDesc = "Task Start";

                        while (true)
                        {

                            byte[] byData = new byte[2048];
                            string srSenderName = "";
                            int inMsgNo = 0;
                            // コレクタで格納されたメールボックスから値を取得
                            //   受信失敗時に空ループ防止用にスリープしているため、ここではタイムアウトなしで
                            //   メール受信を行う。
                            inRet = m_EgApi.EgMailboxReceive(m_EgApi.EGFW.clctRefList[0].Name, ref srSenderName, ref inMsgNo, ref byData, 0);
                            if (inRet == 0)
                            {
                                // メッセージ受信成功
                                srDesc = "Receive Message";

                                // 初回の場合はレコード構成生成
                                if (dwCount == 0)
                                {
                                    srDesc = "Create Format";

                                    // レコード構成生成
                                    inRet = dbCreateFormat(m_EgApi.EGFW.clctRefList[0].DatasetName);
                                    if (inRet != 0)
                                    {
                                        // レコード構成生成失敗
                                        edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "dbCreateFormat " + inRet.ToString());
                                        // タスク処理を終了
                                        break;
                                    }
                                }

                                srDesc = "Insert Binary";

                                // レコードバイナリ追加
                                inRet = dbInsertBinary(byData);
                                dwCount++;
                            }
                            else
                            {
                                // メッセージ受信失敗
                                srDesc = "Receive error 0x" + inRet.ToString("X8");

                                // 空ループ対策用にスリープ
                                Thread.Sleep(100);

                                // タスク処理を終了
                                break;
                            }
                        }
                    }

                });

            }
            catch (Exception ex)
            {
                edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_WRN, "Failed: " + srDesc + " 0x" + ex.HResult.ToString("X8"));
            }
        }

        /// <summary>
        /// レコード構成追加回数
        /// </summary>
        UInt32 m_dwCreateCount = 0;

        /// <summary>
        /// レコード構成番号
        /// </summary>
        int m_inTagFormatVal = 0;

        /// <summary>
        /// タグ名リスト
        /// </summary>
        List<String> m_lsTagName;

        /// <summary>
        /// タグオフセットリスト
        /// </summary>
        List<UInt16> m_lsTagOffs;

        /// <summary>
        /// タグサイズリスト
        /// </summary>
        List<UInt16> m_lsTagSize;

        /// <summary>
        /// 全タグサイズの合計
        /// </summary>
        UInt16 m_woTagDataSize = 0;

        /// <summary>
        /// レコード構成生成
        /// </summary>
        /// <param name="srDatasetName">データセット名</param>
        /// <returns></returns>
        private int dbCreateFormat(string srDatasetName)
        {
            int inRet = 0;
            int i;

            object obSize;

            String srTagResult = "";
            String srTagFormat = "";

            // 応答結果タグ名の生成
            srTagResult = ServiceRealName + ".SDAT.Result" + m_dwCreateCount.ToString();
            // 応答結果タグの生成
            inRet = m_EgApi.EgTagCreateEx(srTagResult, (ushort)EGDEFINE.egTagDataType.Int32, "", "Response result");
            if (inRet != 0)
            {
                // タグ生成失敗
                return -1;
            }

            // 応答レコード構成番号タグ名の生成
            srTagFormat = ServiceRealName + ".SDAT.Format" + m_dwCreateCount.ToString();
            inRet = m_EgApi.EgTagCreateEx(srTagFormat, (ushort)EGDEFINE.egTagDataType.Int32, "", "Response format number");
            if (inRet != 0)
            {
                // タグ生成失敗
                return -2;
            }

            // レコード構成追加カウントインクリメント
            m_dwCreateCount++;

            // メッセージサイズの計算
            UInt16 woMsgLen = 2;                                    // メッセージサイズの更新：メッセージサイズ
            woMsgLen++;                                             // メッセージサイズの更新：タグ数

            // 応答結果タグの追加
            woMsgLen++;                                             // メッセージサイズの更新：タグレングス
            woMsgLen += (UInt16)srTagResult.Length;                 // メッセージサイズの更新：タグ名

            // 応答レコード構成番号タグの追加
            woMsgLen++;                                             // メッセージサイズの更新：タグレングス
            woMsgLen += (UInt16)srTagFormat.Length;                 // メッセージサイズの更新：タグ名

            // Datasetのリンク情報を保持する構造体
            edge_API.TDSLINK dsLink = new edge_API.TDSLINK();

            m_lsTagName = new List<string>();
            m_lsTagOffs = new List<ushort>();
            m_lsTagSize = new List<ushort>();
            m_woTagDataSize = 0;

            // Datasetからタグの一覧を取得する
            inRet = m_EgApi.EgDatasetGetFirst(srDatasetName, ref dsLink);
            if (inRet == 0 && dsLink.pTagName.Length > 0)
            {
                // リストに追加
                m_lsTagName.Add(dsLink.pTagName);
                m_lsTagOffs.Add(dsLink.offset);
                woMsgLen++;                                             // メッセージサイズの更新：タグレングス
                woMsgLen += (UInt16)dsLink.pTagName.Length;             // メッセージサイズの更新：タグ名

                obSize = new object();
                m_EgApi.EgTagGetProperty(dsLink.pTagName, "Size", ref obSize);
                m_lsTagSize.Add((UInt16)obSize);
                m_woTagDataSize += (UInt16)obSize;

                while (true)
                {
                    // 次のデータを取得
                    inRet = m_EgApi.EgDatasetGetNext(srDatasetName, ref dsLink);
                    if (inRet != 0 || dsLink.pTagName.Length == 0)    //次が無ければ処理終了
                    {
                        inRet = 0;
                        break;
                    }

                    // リストに追加
                    m_lsTagName.Add(dsLink.pTagName);
                    m_lsTagOffs.Add(dsLink.offset);
                    woMsgLen++;                                             // メッセージサイズの更新：タグレングス
                    woMsgLen += (UInt16)dsLink.pTagName.Length;             // メッセージサイズの更新：タグ名
                    obSize = new object();
                    m_EgApi.EgTagGetProperty(dsLink.pTagName, "Size", ref obSize);
                    m_lsTagSize.Add((UInt16)obSize);
                    m_woTagDataSize += (UInt16)obSize;
                }
            }
            else
            {
                // データがおかしい
                return -3;
            }

            // レコード構成メッセージの確保
            UInt16 woAllocSize = (UInt16)(4 + woMsgLen);						// メッセージサイズにメッセージ番号(4バイト)を付加
            byte[] byMsg = new byte[woAllocSize];

            UInt16 woMsgPos = 0;                                                // メッセージ設定位置の初期化

            // メッセージ番号の設定
            UInt32 dwMsgNo = (UInt32)SDBMsgNo.CreateFormat;
            Array.Copy(BitConverter.GetBytes(dwMsgNo), 0, byMsg, woMsgPos, sizeof(UInt32));
            woMsgPos += sizeof(UInt32);                                     // メッセージ設定位置の更新

            // メッセージサイズの設定
            Array.Copy(BitConverter.GetBytes(woMsgLen), 0, byMsg, woMsgPos, sizeof(UInt16));
            woMsgPos += sizeof(UInt16);                                     // メッセージ設定位置の更新

            // タグ数の設定
            byMsg[woMsgPos] = (byte)(2 + m_lsTagName.Count);                        // タグ数 = 応答用タグ情報 + 引数のタグ情報
            woMsgPos += sizeof(byte);                                               // メッセージ設定位置の更新

            // 応答結果タグの追加
            // 応答結果.タグレングスの設定
            byMsg[woMsgPos] = (byte)(srTagResult.Length);
            woMsgPos++;                                                             // メッセージ設定位置の更新
            // 応答結果.タグ名の設定
            Array.Copy(Encoding.GetEncoding("Shift_JIS").GetBytes(srTagResult), 0, byMsg, woMsgPos, srTagResult.Length);
            woMsgPos += (byte)(srTagResult.Length);									// メッセージ設定位置の更新

            // 応答レコード構成番号タグの追加
            byMsg[woMsgPos] = (byte)(srTagFormat.Length);
            woMsgPos++;                                                             // メッセージ設定位置の更新
            // 応答レコード構成番号.タグレングスの設定
            Array.Copy(Encoding.GetEncoding("Shift_JIS").GetBytes(srTagFormat), 0, byMsg, woMsgPos, srTagFormat.Length);
            woMsgPos += (byte)(srTagFormat.Length);									// メッセージ設定位置の更新

            // タグ情報の設定
            for (i = 0; i < m_lsTagName.Count; i++)
            {
                // タグレングスの設定
                byMsg[woMsgPos] = (byte)(m_lsTagName[i].Length);
                woMsgPos++;                                                             // メッセージ設定位置の更新
                // タグ名の設定
                Array.Copy(Encoding.GetEncoding("Shift_JIS").GetBytes(m_lsTagName[i]), 0, byMsg, woMsgPos, m_lsTagName[i].Length);
                woMsgPos += (byte)(m_lsTagName[i].Length);								// メッセージ設定位置の更新
            }

            // メールボックスの送信
            inRet = m_EgApi.EgMailboxSend(m_EgApi.EGFW.ServiceMboxName, m_EgApi.EGFW.ServiceMboxName, (int)SDBMsgNo.CreateFormat, byMsg);

            // メールボックスの送信失敗の場合は終了
            if (inRet != 0) { return -4; }

            // 応答結果の取得
            int inTagResultVal = 0;						// 応答結果の読み込み値
            int inRetryCount = 100;						// リトライ回数(0でタイムアウト)
            while (true)
            {
                object obVal = new object();
                // 応答結果の読み込み
                m_EgApi.EgTagRead(srTagResult, ref obVal);
                // 応答結果判定 ... 取得成功はループ終了
                inTagResultVal = (int)obVal;
                if (0 != inTagResultVal)
                {
                    break;
                }

                // リトライ判定
                inRetryCount--;							// リトライ回数デクリメント
                if (inRetryCount > 0)
                {
                    Thread.Sleep(100);
                }
                else
                {
                    // タイムアウトが発生
                    return -5;
                }
            }

            // 応答結果の判定
            if (inTagResultVal == (int)SDBErr.ResultOk)
            {
                object obVal = new object();
                // レコード構成番号の取得
                m_EgApi.EgTagRead(srTagFormat, ref obVal);
                m_inTagFormatVal = (int)obVal;
            }
            else
            {
                // 応答結果を戻り値にセット
                inRet = inTagResultVal;
            }

            return inRet;
        }


        /// <summary>
        /// レコードバイナリ追加
        /// </summary>
        /// <param name="byData">受信データ</param>
        private int dbInsertBinary(byte[] byData)
        {
            int inRet = 0;
            UInt16 woVal;
            UInt32 dwVal;
            Int32 inVal;
            int i;

            // 加算するヘッダサイズの設定
            // - 0x00 .. 0x03  メッセージ番号
            // - 0x04 .. 0x05  メッセージサイズ
            // - 0x06 .. 0x09  レコード構成番号
            UInt16 woHeadSize = 0x0A;
            // レコードバイナリメッセージの確保
            UInt16 woAllocSize = (UInt16)(m_woTagDataSize + woHeadSize);			// メモリ確保サイズの設定
            byte[] byMsg = new byte[woAllocSize];

            UInt16 woMsgPos = 0;												    // メッセージ設定位置の初期化

            // レコードバイナリメッセージの作成:メッセージ番号
            dwVal = (UInt32)SDBMsgNo.InsertBinary;
            Array.Copy(BitConverter.GetBytes(dwVal), 0, byMsg, woMsgPos, sizeof(UInt32));
            woMsgPos += sizeof(UInt32);                                     // メッセージ設定位置の更新

            // レコードバイナリメッセージの作成:メッセージサイズはメッセージ番号を除いたサイズ
            woVal = (UInt16)(woAllocSize - 4);
            Array.Copy(BitConverter.GetBytes(woVal), 0, byMsg, woMsgPos, sizeof(UInt16));
            woMsgPos += sizeof(UInt16);                                     // メッセージ設定位置の更新

            // レコードバイナリメッセージの作成:レコード構成番号
            inVal = m_inTagFormatVal;
            Array.Copy(BitConverter.GetBytes(inVal), 0, byMsg, woMsgPos, sizeof(Int32));
            woMsgPos += sizeof(Int32);                                      // メッセージ設定位置の更新

            // レコードバイナリメッセージの作成:タグデータ
            byte[] byDst = new byte[128];
            uint dwActSize = 0;
            for (i = 0; i < m_lsTagOffs.Count; i++)
            {
                // バイト配列から指定したTag情報を抽出 EgMakeAddressを用いて抽出用フォーマット作成
                inRet = m_EgApi.EgDecode(ref byDst, byData, m_EgApi.EgMakeAddress(m_lsTagOffs[i], (short)m_lsTagSize[i]), ref dwActSize);
                Array.Copy(byDst, 0, byMsg, woMsgPos, m_lsTagSize[i]);
                woMsgPos += m_lsTagSize[i];                                     // メッセージ設定位置の更新
            }

            // メールボックスの送信
            inRet = m_EgApi.EgMailboxSend(m_EgApi.EGFW.ServiceMboxName, m_EgApi.EGFW.ServiceMboxName, (int)SDBMsgNo.InsertBinary, byMsg);

            return inRet;
        }

        /// <summary>
        /// コレクタ開始
        /// </summary>
        private void StartCollector()
        {
            // メッセージ送信 ... 自身にCOLLECTOR START CMDを送る
            fnSendMyMbox((int)EGDEFINE.EGMSG.EM_COLLECTOR_START_ALL);
        }

        /// <summary>
        /// コレクタ停止
        /// </summary>
        private void StopCollector()
        {
            // メッセージ送信 ... 自身にCOLLECTOR STOP CMDを送る
            fnSendMyMbox((int)EGDEFINE.EGMSG.EM_COLLECTOR_STOP_ALL);
        }

        /// <summary>
        /// 自身のメールボックスにメッセージを送信する
        /// </summary>
        /// <param name="inMsgNo">メッセージNo</param>
        /// <returns></returns>
        public int fnSendMyMbox(int inMsgNo)
        {
            int inRet = 0;

            inRet = m_EgApi.EgMailboxSend(m_EgApi.EGFW.ServiceMboxName, m_EgApi.EGFW.ServiceMboxName, inMsgNo);
            edge_FW._C_EgFW_PrintLog(ServiceRealName, edge_FW.EG_LOG_DBG, "EgMailboxSend " + inMsgNo.ToString());

            return inRet;
        }

    }
}
