﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Windows.Forms;

using RTedge;       // RT-edge system API

namespace EgSDB
{

    /// <summary>
    /// メイン画面
    /// </summary>
    public partial class frmMain : Form
    {
        /// <summary>
        /// 自身のプログラム名
        /// </summary>
        private string m_srMyName = "";

        /// <summary>
        /// 起動引数
        /// </summary>
        private string[] m_srArgs;

        /// <summary>
        /// SDBManager
        /// </summary>
        private SDBManeger m_SDBManger;

        /// <summary>
        /// フォーム非表示(true=フォームを非表示にする)
        /// </summary>
        private bool m_boFromHide = true;

        /// <summary>
        /// 最新のエラーコード取得
        /// </summary>
        /// <returns>最新のエラーコード</returns>
        public Int32 fnGetLastErr()
        {
            return m_SDBManger.fnGetLastErr();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="srMyName">自身のプログラム名</param>
        public frmMain(string srMyName)
        {

            // コンポーネントの初期化
            InitializeComponent();

            // 自身のプログラム名の設定
            m_srMyName = srMyName;

            // SDB Manager初期化
            m_SDBManger = new SDBManeger(srMyName);

            // 起動引数の取得
            m_srArgs = Environment.GetCommandLineArgs();

            // フォームの非表示設定
            fnHideForm();

        }

        /// <summary>
        /// フォームの非表示設定 
        /// </summary>
        private void fnHideForm()
        {

            // 取得起動引数ループ
            foreach (string srArg in m_srArgs)
            {
                // 起動引数"ShowForm"が設定されているか？
                if ("ShowForm" == srArg)
                {
                    m_boFromHide = false;                   // フォームを表示する。
                    break;                                  // ループの終了
                }
            }

            // フォームの非表示表示判定
            if (true == m_boFromHide)
            {
                // フォームの非表示
                ShowInTaskbar = false;                      // タスクバーに表示しない
                WindowState = FormWindowState.Minimized;    // 最小化で起動(タスクトレイに入る)
            }

        }

        /// <summary>
        /// フォームロード
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void frmMain_Load(object sender, EventArgs e)
        {
            // フォーム表示の場合は終了
            if (false == m_boFromHide) {
                return;
            }

            int inRet = (int)SDBErr.Ok;                     // 戻り値

            // 初期化
            inRet = fnInitSDB();
            if ((int)SDBErr.Ok != inRet)
            {
                MessageBox.Show("Failed: initial SDB. " + inRet.ToString() + " (0x" + inRet.ToString("X8") + ")", m_srMyName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // 自動起動設定無効の場合は終了
            if (false == m_SDBManger.fnGetAutoRun()) { return; }

            // 実行
            inRet = fnRunSDB();
            if ((int)SDBErr.Ok != inRet)
            {
                MessageBox.Show("Failed: run SDB. " + inRet.ToString() + " (0x" + inRet.ToString("X8") + ")", m_srMyName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // 自身のフォームを隠す
            this.Hide();
        }

        /// <summary>
        /// フォームクロージング
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 停止
            fnStopSDB();
            // メッセージハンドラでアプリケーションを終了させるためフォームを閉じない
            e.Cancel = true;
        }

        /// <summary>
        /// [Init]クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnInit_Click(object sender, EventArgs e)
        {
            fnInitSDB();
        }

        /// <summary>
        /// 初期化
        /// </summary>
        /// <returns></returns>
        private int fnInitSDB()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値

            // 初期化
            inRet = m_SDBManger.InitEdge();
            if (inRet != (Int32)SDBErr.Ok)
            {
                // 初期化失敗
                return inRet;
            }

            // ファイルリストの生成
            inRet = m_SDBManger.CreateList();
            if (inRet != (Int32)SDBErr.Ok)
            {
                // ファイルリストの生成エラー
                return inRet;
            }

            // ファイルをオープン
            inRet = m_SDBManger.OpenFile();
            if (inRet > 0)
            {
                // オープン成功
                inRet = (Int32)SDBErr.Ok;
            }

            return inRet;

        }

        /// <summary>
        /// [Exit]クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExit_Click(object sender, EventArgs e)
        {
            fnExitSDB();
        }

        /// <summary>
        /// 終了
        /// </summary>
        /// <returns></returns>
        private int fnExitSDB()
        {
            int inRet = (int)SDBErr.Ok;                     // 戻り値

            // ファイルをクローズ
            inRet = m_SDBManger.CloseFile(SDBManeger.CloseFileFactor.ExitContainer);

            return inRet;
        }

        /// <summary>
        /// [Run]クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRun_Click(object sender, EventArgs e)
        {
            fnRunSDB();
        }

        /// <summary>
        /// 実行
        /// </summary>
        /// <returns></returns>
        private int fnRunSDB()
        {
            return m_SDBManger.fnSendMyMbox((int)EGDEFINE.EGMSG.EM_SERVICE_RUN);
        }

        /// <summary>
        /// [Pause]クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPause_Click(object sender, EventArgs e)
        {
            fnPauseSDB();
        }

        /// <summary>
        /// 一時停止
        /// </summary>
        /// <returns></returns>
        private int fnPauseSDB()
        {
            return m_SDBManger.fnSendMyMbox((int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE);
        }

        /// <summary>
        /// [Stop]クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStop_Click(object sender, EventArgs e)
        {
            fnStopSDB();
        }

        /// <summary>
        /// 停止
        /// </summary>
        /// <returns></returns>
        private int fnStopSDB()
        {
            return m_SDBManger.fnSendMyMbox((int)EGDEFINE.EGMSG.EM_SERVICE_STOP);
        }

    }

}
